package aood.calcsheet.dbjp;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Sheet is a matrix of Long values.
 * They can be accessed through the certain column 
 * (see {@link #getColumn(int)}). Sheet size can be
 * determined through {@link #getWidth()} and {@link #getHeight()}.<br>
 * 
 * A sheet can be loaded from the file using {@link #load(String)}, or
 * created empty with specified size (see {@link #Sheet(int, int)}.<br>
 * 
 * @author Daniel Badtke, Jacek Pospychała
 *
 */
public class Sheet {
	/**
	 * Default row fields separator in input and output file.  
	 */
	public static final String FIELD_SEPARATOR = ",";
	
	/**
	 * Internal cells data storage.
	 */
	private List<List<Long>> columns;

	/**
	 * Default constructor, useful with {@link #load(String)} method,
	 * as it creates an empty sheet.
	 *
	 */
	public Sheet() {
		columns = new ArrayList<List<Long>>();
	}
	
	/**
	 * Create custom size sheet. Created with this method, all
	 * fields have value set to 0.
	 * 
	 * @param width number of columns of new sheet.
	 * @param height number of rows of new sheet.
	 */
	public Sheet(int width, int height) {
		columns = new ArrayList<List<Long>>(width);
		int col;
		int row;
		
		for (col=0; col<width; col++) {
			ArrayList<Long> emptyColumn = new ArrayList<Long>(height);
			for (row=0; row<height; row++) {
				emptyColumn.add(0L);
			}
			columns.add(emptyColumn);
		}
	}
	
	/**
	 * Set value in sheet's certain cell. If col or row
	 * exceeds sheet size, method will throw {@link IndexOutOfBoundsException}.
	 * 
	 * @param col changed cell column number.
	 * @param row changed cell row number.
	 * @param value new value to be set.
	 * @throws IndexOutOfBoundsException if col or row exceeds sheet size.
	 */
	public void set(int col, int row, Long value) {
		List<Long> column = columns.get(col);
		column.set(row, value);
	}
	
	/**
	 * Returns a column containing Long values of given number.
	 * 
	 * @param col column number.
	 * @return an unmodifiable column with Long values.
	 */
	public List<Long> getColumn(int col) {
		return Collections.unmodifiableList(columns.get(col));
	}
	
	public int getWidth() {
		return columns.size();
	}
	
	public int getHeight() {
		return (columns.size() > 0) ? columns.get(0).size() : 0;
	}
	
	/**
	 * Adds a value to the end of column with a specified number.
	 * If (col &lt; getWidth()), method will create all necessary columns.
	 *   
	 * @param col column number, where value should be added.
	 * @param value value to be added.
	 */
	private void add(int col, Long value) {
		if (col < getWidth()) {
			columns.get(col).add(value);
		} else {
			columns.add(new ArrayList<Long>());
			add(col, value);
		}
	}
	
	/**
	 * Parse a string and add long values delimited with {@link #FIELD_SEPARATOR}
	 * to the ends of the columns.
	 * 
	 * @param line A String with delimited long values to be parsed
	 */
	private void parseRow(String line) {
		StringTokenizer st = new StringTokenizer(line, FIELD_SEPARATOR);
		int colNum = 0;
		
		while (st.hasMoreTokens()) {
			Long value = Long.parseLong(st.nextToken());
			add(colNum, value);
			colNum++;
		}
	}

	/**
	 * Loads a long values from given file to the sheet.
	 * Input file contains lines with long numbers delimited
	 * with FIELD_SEPARATOR.
	 * 
	 * @param filename a name of file to be read.
	 * @throws IOException if the file doesn't exist, or an error
	 * occurs while reading from it.
	 * @throws NumberFormatException if some number cannot be parsed.
	 */
	public void load(String filename) throws IOException {
		FileInputStream fstream = new FileInputStream(filename);
		BufferedReader in = new BufferedReader(
				new InputStreamReader(fstream));
		String line;
		
		columns.clear();
		try 
		{
			while ((line = in.readLine()) != null) {
				parseRow(line);
			}
		} finally {
			in.close();
		}
	}
	
	/**
	 * Saves the sheet to the file with given name. File format
	 * is the same as recognized by {@link #load(String)} method.
	 * 
	 * @param filename Name of output file.
	 * @throws IOException if file can't be opened for writing, or an
	 * error occurs while writing to file.
	 */
	public void save(String filename) throws IOException {
		FileOutputStream fstream = new FileOutputStream(filename);
		PrintWriter out = new PrintWriter(fstream);
		int rowNum = 0;
		int colNum;
		
		try {
			for (rowNum = 0; rowNum < getHeight(); rowNum++) {
				for (colNum=0; colNum < getWidth() -1; colNum++) {
					List<Long> column = getColumn(colNum);
					out.print(column.get(rowNum) + FIELD_SEPARATOR);
				}
				out.println(getColumn(getWidth() - 1).get(rowNum));
			}
		} finally {
			out.close();
		}
	}

}
